@@ -17,26 +17,69 @@
  */
 
 #include <assert.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
-#include <cgraph/exit.h>
-#include <cgraph/gv_math.h>
-#include <cgraph/list.h>
-#include <cgraph/queue.h>
 #include <dotgen/dot.h>
+#include <inttypes.h>
 #include <limits.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/gv_math.h>
+#include <util/itos.h>
+#include <util/list.h>
 #include <util/streq.h>
 
 struct adjmatrix_t {
   size_t nrows;
   size_t ncols;
-  char *data;
+  uint8_t *data;    ///< bit-packed backing memory
+  size_t allocated; ///< how many bytes have been allocated backing `data`?
 };
 
+/// get the value of a matrix cell
+///
+/// @param me Matrix to inspect
+/// @param row Row coordinate
+/// @param col Column coordinate
+/// @return True if the given cell was set
+static bool matrix_get(adjmatrix_t *me, size_t row, size_t col) {
+  assert(me != NULL);
+
+  // if this index is beyond anything set, infer it as unset
+  const size_t index = row * me->ncols + col;
+  const size_t byte_index = index / 8;
+  const size_t bit_index = index % 8;
+  if (byte_index >= me->allocated) {
+    return false;
+  }
+
+  return (me->data[byte_index] >> bit_index) & 1;
+}
+
+/// set the value of a matrix cell to true
+///
+/// @param me Matrix to update
+/// @param row Row coordinate
+/// @param col Column coordinate
+static void matrix_set(adjmatrix_t *me, size_t row, size_t col) {
+  assert(me != NULL);
+
+  // if we are updating beyond allocated space, expand the backing store
+  const size_t index = row * me->ncols + col;
+  const size_t byte_index = index / 8;
+  const size_t bit_index = index % 8;
+  if (byte_index >= me->allocated) {
+    me->data = gv_recalloc(me->data, me->allocated, byte_index + 1,
+                           sizeof(me->data[0]));
+    me->allocated = byte_index + 1;
+  }
+
+  me->data[byte_index] |= (uint8_t)(UINT8_C(1) << bit_index);
+}
+
 /* #define DEBUG */
 #define MARK(v)		(ND_mark(v))
 #define saveorder(v)	(ND_coord(v)).x
@@ -52,9 +95,11 @@ static void flat_search(graph_t * g, node_t * v);
 static void init_mincross(graph_t * g);
 static void merge2(graph_t * g);
 static void init_mccomp(graph_t *g, size_t c);
-static void cleanup2(graph_t * g, int nc);
-static int mincross_clust(graph_t *g, ints_t *scratch);
-static int mincross(graph_t *g, int startpass, ints_t *scratch);
+static void cleanup2(graph_t *g, int64_t nc);
+/// @return minimum crossings on success, negative value on failure
+static int64_t mincross_clust(graph_t *g, ints_t *scratch);
+/// @return minimum crossings on success, negative value on failure
+static int64_t mincross(graph_t *g, int startpass, ints_t *scratch);
 static void mincross_step(graph_t * g, int pass);
 static void mincross_options(graph_t * g);
 static void save_best(graph_t * g);
@@ -62,14 +107,8 @@ static void restore_best(graph_t * g);
 static adjmatrix_t *new_matrix(size_t i, size_t j);
 static void free_matrix(adjmatrix_t * p);
 static int ordercmpf(const void *, const void *);
-static int ncross(ints_t *scratch);
+static int64_t ncross(ints_t *scratch);
 #ifdef DEBUG
-#if DEBUG > 1
-static int gd_minrank(Agraph_t *g) {return GD_minrank(g);}
-static int gd_maxrank(Agraph_t *g) {return GD_maxrank(g);}
-static rank_t *gd_rank(Agraph_t *g, int r) {return &GD_rank(g)[r];}
-static int nd_order(Agnode_t *v) { return ND_order(v); }
-#endif
 void check_rs(graph_t * g, int null_ok);
 void check_order(void);
 void check_vlists(graph_t * g);
@@ -96,17 +135,15 @@ static void indent(graph_t* g)
   }
 }
 
-static char* nname(node_t* v)
-{
-        static char buf[1000];
+/// @param stream Output stream to write to
+static void nname(node_t *v, FILE *stream) {
 	if (ND_node_type(v)) {
 		if (ND_ranktype(v) == CLUSTER)
-			snprintf(buf, sizeof(buf), "v%s_%p", agnameof(ND_clust(v)), v);
+			fprintf(stream, "v%s_%p", agnameof(ND_clust(v)), v);
 		else
-			snprintf(buf, sizeof(buf), "v_%p", v);
+			fprintf(stream, "v_%p", v);
 	} else
-		snprintf(buf, sizeof(buf), "%s", agnameof(v));
-	return buf;
+		fputs(agnameof(v), stream);
 }
 static void dumpg (graph_t* g)
 {
@@ -120,9 +157,8 @@ static void dumpg (graph_t* g)
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
 	  v = GD_rank(g)[r].v[i];
           if (i > 0)
- 	    fprintf (stderr, " -> %s", nname(v));
-          else
- 	    fprintf (stderr, "%s", nname(v));
+ 	    fputs(" -> ", stderr);
+          nname(v, stderr);
         }
         if (i > 1) fprintf (stderr, " [style=invis]}\n");
         else fprintf (stderr, " }\n");
@@ -131,8 +167,10 @@ static void dumpg (graph_t* g)
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
 	  v = GD_rank(g)[r].v[i];
 	  for (j = 0; (e = ND_out(v).list[j]); j++) {
-             fprintf (stderr, "%s -> ", nname(v));
-             fprintf (stderr, "%s\n", nname(aghead(e)));
+             nname(v, stderr);
+             fputs(" -> ", stderr);
+             nname(aghead(e), stderr);
+             fputc('\n', stderr);
           }
         }
     }
@@ -148,7 +186,8 @@ static void dumpr (graph_t* g, int edges)
 	fprintf (stderr, "[%d] ", r);
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
 	  v = GD_rank(g)[r].v[i];
- 	  fprintf (stderr, "%s(%.02f,%d) ", nname(v), saveorder(v),ND_order(v));
+ 	  nname(v, stderr);
+ 	  fprintf(stderr, "(%.02f,%d) ", saveorder(v),ND_order(v));
         }
 	fprintf (stderr, "\n");
     }
@@ -157,8 +196,10 @@ static void dumpr (graph_t* g, int edges)
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
 	  v = GD_rank(g)[r].v[i];
 	  for (j = 0; (e = ND_out(v).list[j]); j++) {
-             fprintf (stderr, "%s -> ", nname(v));
-             fprintf (stderr, "%s\n", nname(aghead(e)));
+             nname(v, stderr);
+             fputs(" -> ", stderr);
+             nname(aghead(e), stderr);
+             fputc('\n', stderr);
           }
         }
     }
@@ -242,9 +283,7 @@ getComp (graph_t* g, node_t* n, graph_t* comp, int* indices)
     return backedge;
 }
 
-/* fixLabelOrder:
- * For each pair of nodes (labels), we add an edge 
- */
+/// for each pair of nodes (labels), we add an edge 
 static void
 fixLabelOrder (graph_t* g, rank_t* rk)
 {
@@ -291,8 +330,7 @@ fixLabelOrder (graph_t* g, rank_t* rk)
     free (arr);
 }
 
-/* checkLabelOrder:
- * Check that the ordering of labels for flat edges is consistent. 
+/* Check that the ordering of labels for flat edges is consistent. 
  * This is necessary because dot_position will attempt to force the label
  * to be between the edge's vertices. This can lead to an infeasible problem.
  *
@@ -305,7 +343,6 @@ checkLabelOrder (graph_t* g)
 {
     int j, r, lo, hi;
     graph_t* lg = NULL;
-    agxbuf buf = {0};
     rank_t* rk;
     Agnode_t* u;
     Agnode_t* n;
@@ -317,16 +354,12 @@ checkLabelOrder (graph_t* g)
 	    u = rk->v[j];
 	    if ((e = ND_alg(u))) {
 		if (!lg) lg = agopen ("lg", Agstrictdirected, 0);
-		agxbprint(&buf, "%d", j);
-		n = agnode(lg, agxbuse(&buf), 1);
+		n = agnode(lg, ITOS(j), 1);
 		agbindrec(n, "info", sizeof(info_t), true);
 		lo = ND_order(aghead(ND_out(u).list[0]));
 		hi = ND_order(aghead(ND_out(u).list[1]));
 		if (lo > hi) {
-		    int tmp;
-		    tmp = lo;
-		    lo = hi;
-		    hi = tmp;
+		    SWAP(&lo, &hi);
 		}
 		ND_lo(n) = lo;
 		ND_hi(n) = hi;
@@ -339,16 +372,14 @@ checkLabelOrder (graph_t* g)
 	    lg = NULL;
 	}
     }
-    agxbfree(&buf);
 }
 
-/* dot_mincross:
- * Minimize edge crossings
+/* Minimize edge crossings
  * Note that nodes are not placed into GD_rank(g) until mincross()
  * is called.
  */
-void dot_mincross(graph_t *g) {
-    int nc;
+int dot_mincross(graph_t *g) {
+    int64_t nc;
     char *s;
 
     /* check whether malformed input has led to empty cluster that the crossing
@@ -375,14 +406,24 @@ void dot_mincross(graph_t *g) {
     size_t comp;
     for (nc = 0, comp = 0; comp < GD_comp(g).size; comp++) {
 	init_mccomp(g, comp);
-	nc += mincross(g, 0, &scratch);
+	const int64_t mc = mincross(g, 0, &scratch);
+	if (mc < 0) {
+	    ints_free(&scratch);
+	    return -1;
+	}
+	nc += mc;
     }
 
     merge2(g);
 
     /* run mincross on contents of each cluster */
     for (int c = 1; c <= GD_n_cluster(g); c++) {
-	nc += mincross_clust(GD_clust(g)[c], &scratch);
+	const int64_t mc = mincross_clust(GD_clust(g)[c], &scratch);
+	if (mc < 0) {
+	    ints_free(&scratch);
+	    return -1;
+	}
+	nc += mc;
 #ifdef DEBUG
 	check_vlists(GD_clust(g)[c]);
 	check_order();
@@ -392,7 +433,12 @@ void dot_mincross(graph_t *g) {
     if (GD_n_cluster(g) > 0 && (!(s = agget(g, "remincross")) || mapbool(s))) {
 	mark_lowclusters(g);
 	ReMincross = true;
-	nc = mincross(g, 2, &scratch);
+	const int64_t mc = mincross(g, 2, &scratch);
+	if (mc < 0) {
+	    ints_free(&scratch);
+	    return -1;
+	}
+	nc = mc;
 #ifdef DEBUG
 	for (int c = 1; c <= GD_n_cluster(g); c++)
 	    check_vlists(GD_clust(g)[c]);
@@ -400,13 +446,13 @@ void dot_mincross(graph_t *g) {
     }
     ints_free(&scratch);
     cleanup2(g, nc);
+    return 0;
 }
 
 static adjmatrix_t *new_matrix(size_t i, size_t j) {
     adjmatrix_t *rv = gv_alloc(sizeof(adjmatrix_t));
     rv->nrows = i;
     rv->ncols = j;
-    rv->data = gv_calloc(i * j, sizeof(char));
     return rv;
 }
 
@@ -418,8 +464,6 @@ static void free_matrix(adjmatrix_t * p)
     }
 }
 
-#define ELT(M,i,j)		(M->data[((i)*M->ncols)+(j)])
-
 static void init_mccomp(graph_t *g, size_t c) {
     int r;
 
@@ -506,8 +550,7 @@ static void do_ordering_for_nodes(graph_t * g)
     }
 }
 
-/* ordered_edges:
- * handle case where graph specifies edge ordering
+/* handle case where graph specifies edge ordering
  * If the graph does not have an ordering attribute, we then
  * check for nodes having the attribute.
  * Note that, in this implementation, the value of G_ordering
@@ -540,25 +583,33 @@ static void ordered_edges(graph_t * g)
     }
 }
 
-static int mincross_clust(graph_t *g, ints_t *scratch) {
-    int c, nc;
+static int64_t mincross_clust(graph_t *g, ints_t *scratch) {
+    int c;
 
-    expand_cluster(g);
+    if (expand_cluster(g) != 0) {
+	return -1;
+    }
     ordered_edges(g);
     flat_breakcycles(g);
     flat_reorder(g);
-    nc = mincross(g, 2, scratch);
+    int64_t nc = mincross(g, 2, scratch);
+    if (nc < 0) {
+	return nc;
+    }
 
-    for (c = 1; c <= GD_n_cluster(g); c++)
-	nc += mincross_clust(GD_clust(g)[c], scratch);
+    for (c = 1; c <= GD_n_cluster(g); c++) {
+	const int64_t mc = mincross_clust(GD_clust(g)[c], scratch);
+	if (mc < 0) {
+	    return mc;
+	}
+	nc += mc;
+    }
 
     save_vlist(g);
     return nc;
 }
 
 static bool left2right(graph_t *g, node_t *v, node_t *w) {
-    adjmatrix_t *M;
-
     /* CLUSTER indicates orig nodes of clusters, and vnodes of skeletons */
     if (!ReMincross) {
 	if (ND_clust(v) != ND_clust(w) && ND_clust(v) && ND_clust(w)) {
@@ -573,21 +624,19 @@ static bool left2right(graph_t *g, node_t *v, node_t *w) {
 	if (ND_clust(v) != ND_clust(w))
 	    return true;
     }
-    M = GD_rank(g)[ND_rank(v)].flat;
+    adjmatrix_t *const M = GD_rank(g)[ND_rank(v)].flat;
     if (M == NULL)
 	return false;
     if (GD_flip(g)) {
-	node_t *t = v;
-	v = w;
-	w = t;
+	SWAP(&v, &w);
     }
-    return ELT(M, flatindex(v), flatindex(w)) != 0;
+    return matrix_get(M, (size_t)flatindex(v), (size_t)flatindex(w));
 }
 
-static int in_cross(node_t * v, node_t * w)
-{
+static int64_t in_cross(node_t *v, node_t *w) {
     edge_t **e1, **e2;
-    int inv, cross = 0, t;
+    int inv, t;
+    int64_t cross = 0;
 
     for (e2 = ND_in(w).list; *e2; e2++) {
 	int cnt = ED_xpenalty(*e2);		
@@ -635,12 +684,11 @@ static void exchange(node_t * v, node_t * w)
     GD_rank(Root)[r].v[vi] = w;
 }
 
-static int transpose_step(graph_t * g, int r, bool reverse)
-{
-    int i, c0, c1, rv;
+static int64_t transpose_step(graph_t *g, int r, bool reverse) {
+    int i;
     node_t *v, *w;
 
-    rv = 0;
+    int64_t rv = 0;
     GD_rank(g)[r].candidate = false;
     for (i = 0; i < GD_rank(g)[r].n - 1; i++) {
 	v = GD_rank(g)[r].v[i];
@@ -648,7 +696,8 @@ static int transpose_step(graph_t * g, int r, bool reverse)
 	assert(ND_order(v) < ND_order(w));
 	if (left2right(g, v, w))
 	    continue;
-	c0 = c1 = 0;
+	int64_t c0 = 0;
+	int64_t c1 = 0;
 	if (r > 0) {
 	    c0 += in_cross(v, w);
 	    c1 += in_cross(w, v);
@@ -678,10 +727,11 @@ static int transpose_step(graph_t * g, int r, bool reverse)
 
 static void transpose(graph_t * g, bool reverse)
 {
-    int r, delta;
+    int r;
 
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++)
 	GD_rank(g)[r].candidate = true;
+    int64_t delta;
     do {
 	delta = 0;
 	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
@@ -692,21 +742,23 @@ static void transpose(graph_t * g, bool reverse)
     } while (delta >= 1);
 }
 
-static int mincross(graph_t *g, int startpass, ints_t *scratch) {
+static int64_t mincross(graph_t *g, int startpass, ints_t *scratch) {
     const int endpass = 2;
     int maxthispass = 0, iter, trying, pass;
-    int cur_cross, best_cross;
+    int64_t cur_cross, best_cross;
 
     if (startpass > 1) {
 	cur_cross = best_cross = ncross(scratch);
 	save_best(g);
     } else
-	cur_cross = best_cross = INT_MAX;
+	cur_cross = best_cross = INT64_MAX;
     for (pass = startpass; pass <= endpass; pass++) {
 	if (pass <= 1) {
 	    maxthispass = MIN(4, MaxIter);
 	    if (g == dot_root(g))
-		build_ranks(g, pass, scratch);
+		if (build_ranks(g, pass, scratch) != 0) {
+		    return -1;
+		}
 	    if (pass == 0)
 		flat_breakcycles(g);
 	    flat_reorder(g);
@@ -725,7 +777,8 @@ static int mincross(graph_t *g, int startpass, ints_t *scratch) {
 	for (iter = 0; iter < maxthispass; iter++) {
 	    if (Verbose)
 		fprintf(stderr,
-			"mincross: pass %d iter %d trying %d cur_cross %d best_cross %d\n",
+			"mincross: pass %d iter %d trying %d cur_cross %" PRId64 " best_cross %"
+			PRId64 "\n",
 			pass, iter, trying, cur_cross, best_cross);
 	    if (trying++ >= MinQuit)
 		break;
@@ -734,7 +787,7 @@ static int mincross(graph_t *g, int startpass, ints_t *scratch) {
 	    mincross_step(g, iter);
 	    if ((cur_cross = ncross(scratch)) <= best_cross) {
 		save_best(g);
-		if (cur_cross < Convergence * best_cross)
+		if (cur_cross < Convergence * (double)best_cross)
 		    trying = 0;
 		best_cross = cur_cross;
 	    }
@@ -834,8 +887,7 @@ static void merge2(graph_t * g)
     }
 }
 
-static void cleanup2(graph_t * g, int nc)
-{
+static void cleanup2(graph_t *g, int64_t nc) {
     int i, j, r, c;
     node_t *v;
     edge_t *e;
@@ -870,22 +922,20 @@ static void cleanup2(graph_t * g, int nc)
 	free_matrix(GD_rank(g)[r].flat);
     }
     if (Verbose)
-	fprintf(stderr, "mincross %s: %d crossings, %.2f secs.\n",
+	fprintf(stderr, "mincross %s: %" PRId64 " crossings, %.2f secs.\n",
 		agnameof(g), nc, elapsed_sec());
 }
 
 static node_t *neighbor(node_t * v, int dir)
 {
-    node_t *rv;
-
-    rv = NULL;
+    node_t *rv = NULL;
 assert(v);
     if (dir < 0) {
 	if (ND_order(v) > 0)
 	    rv = GD_rank(Root)[ND_rank(v)].v[ND_order(v) - 1];
     } else
 	rv = GD_rank(Root)[ND_rank(v)].v[ND_order(v) + 1];
-assert((rv == 0) || (ND_order(rv)-ND_order(v))*dir > 0);
+assert(rv == 0 || (ND_order(rv)-ND_order(v))*dir > 0);
     return rv;
 }
 
@@ -911,10 +961,8 @@ static bool inside_cluster(graph_t *g, node_t *v) {
 
 static node_t *furthestnode(graph_t * g, node_t * v, int dir)
 {
-    node_t *u, *rv;
-
-    rv = u = v;
-    while ((u = neighbor(u, dir))) {
+    node_t *rv = v;
+    for (node_t *u = v; (u = neighbor(u, dir)); ) {
 	if (is_a_normal_node_of(g, u))
 	    rv = u;
 	else if (is_a_vnode_of_an_edge_of(g, u))
@@ -945,21 +993,21 @@ void rec_save_vlists(graph_t * g)
 
 void rec_reset_vlists(graph_t * g)
 {
-    int r, c;
-    node_t *u, *v, *w;
-
-    /* fix vlists of sub-clusters */
-    for (c = 1; c <= GD_n_cluster(g); c++)
+    // fix vlists of sub-clusters
+    for (int c = 1; c <= GD_n_cluster(g); c++)
 	rec_reset_vlists(GD_clust(g)[c]);
 
     if (GD_rankleader(g))
-	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	    v = GD_rankleader(g)[r];
+	for (int r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+	    node_t *const v = GD_rankleader(g)[r];
+	    if (v == NULL) {
+	        continue;
+	    }
 #ifdef DEBUG
 	    node_in_root_vlist(v);
 #endif
-	    u = furthestnode(g, v, -1);
-	    w = furthestnode(g, v, 1);
+	    node_t *const u = furthestnode(g, v, -1);
+	    node_t *const w = furthestnode(g, v, 1);
 	    GD_rankleader(g)[r] = u;
 #ifdef DEBUG
 	    assert(GD_rank(dot_root(g))[r].v[ND_order(u)] == u);
@@ -969,8 +1017,7 @@ void rec_reset_vlists(graph_t * g)
 	}
 }
 
-/* realFillRanks:
- * The structures in crossing minimization and positioning require
+/* The structures in crossing minimization and positioning require
  * that clusters have some node on each rank. This function recursively
  * guarantees this property. It takes into account nodes and edges in
  * a cluster, the latter causing dummy nodes for intervening ranks.
@@ -1099,7 +1146,7 @@ static void flat_search(graph_t * g, node_t * v)
 	    if (ND_onstack(aghead(e))) {
 		assert(flatindex(aghead(e)) < M->nrows);
 		assert(flatindex(agtail(e)) < M->ncols);
-		ELT(M, flatindex(aghead(e)), flatindex(agtail(e))) = 1;
+		matrix_set(M, (size_t)flatindex(aghead(e)), (size_t)flatindex(agtail(e)));
 		delete_flat_edge(e);
 		i--;
 		if (ED_edge_type(e) == FLATORDER)
@@ -1108,7 +1155,7 @@ static void flat_search(graph_t * g, node_t * v)
 	    } else {
 		assert(flatindex(aghead(e)) < M->nrows);
 		assert(flatindex(agtail(e)) < M->ncols);
-		ELT(M, flatindex(agtail(e)), flatindex(aghead(e))) = 1;
+		matrix_set(M, (size_t)flatindex(agtail(e)), (size_t)flatindex(aghead(e)));
 		if (!ND_mark(aghead(e)))
 		    flat_search(g, aghead(e));
 	    }
@@ -1118,20 +1165,20 @@ static void flat_search(graph_t * g, node_t * v)
 
 static void flat_breakcycles(graph_t * g)
 {
-    int i, r, flat;
+    int i, r;
     node_t *v;
 
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	flat = 0;
+	bool flat = false;
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
 	    v = GD_rank(g)[r].v[i];
 	    ND_mark(v) = false;
 	    ND_onstack(v) = false;
 	    ND_low(v) = i;
-	    if (ND_flat_out(v).size > 0 && flat == 0) {
+	    if (ND_flat_out(v).size > 0 && !flat) {
 		GD_rank(g)[r].flat =
 		    new_matrix((size_t)GD_rank(g)[r].n, (size_t)GD_rank(g)[r].n);
-		flat = 1;
+		flat = true;
 	    }
 	}
 	if (flat) {
@@ -1144,8 +1191,7 @@ static void flat_breakcycles(graph_t * g)
     }
 }
 
-/* allocate_ranks:
- * Allocate rank structure, determining number of nodes per rank.
+/* Allocate rank structure, determining number of nodes per rank.
  * Note that no nodes are put into the structure yet.
  */
 void allocate_ranks(graph_t * g)
@@ -1161,9 +1207,7 @@ void allocate_ranks(graph_t * g)
 	    low = ND_rank(agtail(e));
 	    high = ND_rank(aghead(e));
 	    if (low > high) {
-		int t = low;
-		low = high;
-		high = t;
+		SWAP(&low, &high);
 	    }
 	    for (r = low + 1; r < high; r++)
 		cn[r]++;
@@ -1178,8 +1222,7 @@ void allocate_ranks(graph_t * g)
 }
 
 /* install a node at the current right end of its rank */
-void install_in_rank(graph_t * g, node_t * n)
-{
+int install_in_rank(graph_t *g, node_t *n) {
     int i, r;
 
     r = ND_rank(n);
@@ -1187,7 +1230,7 @@ void install_in_rank(graph_t * g, node_t * n)
     if (GD_rank(g)[r].an <= 0) {
 	agerrorf("install_in_rank, line %d: %s %s rank %d i = %d an = 0\n",
 	      __LINE__, agnameof(g), agnameof(n), r, i);
-	return;
+	return -1;
     }
 
     GD_rank(g)[r].v[i] = n;
@@ -1207,30 +1250,31 @@ void install_in_rank(graph_t * g, node_t * n)
     if (ND_order(n) > GD_rank(Root)[r].an) {
 	agerrorf("install_in_rank, line %d: ND_order(%s) [%d] > GD_rank(Root)[%d].an [%d]\n",
 	      __LINE__, agnameof(n), ND_order(n), r, GD_rank(Root)[r].an);
-	return;
+	return -1;
     }
     if (r < GD_minrank(g) || r > GD_maxrank(g)) {
 	agerrorf("install_in_rank, line %d: rank %d not in rank range [%d,%d]\n",
 	      __LINE__, r, GD_minrank(g), GD_maxrank(g));
-	return;
+	return -1;
     }
     if (GD_rank(g)[r].v + ND_order(n) >
 	GD_rank(g)[r].av + GD_rank(Root)[r].an) {
 	agerrorf("install_in_rank, line %d: GD_rank(g)[%d].v + ND_order(%s) [%d] > GD_rank(g)[%d].av + GD_rank(Root)[%d].an [%d]\n",
 	      __LINE__, r, agnameof(n),ND_order(n), r, r, GD_rank(Root)[r].an);
-	return;
+	return -1;
     }
+    return 0;
 }
 
 /*	install nodes in ranks. the initial ordering ensure that series-parallel
  *	graphs such as trees are drawn with no crossings.  it tries searching
  *	in- and out-edges and takes the better of the two initial orderings.
  */
-void build_ranks(graph_t *g, int pass, ints_t *scratch) {
+int build_ranks(graph_t *g, int pass, ints_t *scratch) {
     int i, j;
-    node_t *n, *n0, *ns;
+    node_t *n, *ns;
     edge_t **otheredges;
-    queue_t q = {0};
+    node_queue_t q = {0};
     for (n = GD_nlist(g); n; n = ND_next(n))
 	MARK(n) = false;
 
@@ -1265,18 +1309,26 @@ void build_ranks(graph_t *g, int pass, ints_t *scratch) {
 	    continue;
 	if (!MARK(n)) {
 	    MARK(n) = true;
-	    queue_push(&q, n);
-	    while ((n0 = queue_pop(&q))) {
+	    node_queue_push_back(&q, n);
+	    while (!node_queue_is_empty(&q)) {
+		node_t *n0 = node_queue_pop_front(&q);
 		if (ND_ranktype(n0) != CLUSTER) {
-		    install_in_rank(g, n0);
+		    if (install_in_rank(g, n0) != 0) {
+		        node_queue_free(&q);
+		        return -1;
+		    }
 		    enqueue_neighbors(&q, n0, pass);
 		} else {
-		    install_cluster(g, n0, pass, &q);
+		    const int rc = install_cluster(g, n0, pass, &q);
+		    if (rc != 0) {
+		        node_queue_free(&q);
+		        return rc;
+		    }
 		}
 	    }
 	}
     }
-    assert(queue_pop(&q) == NULL);
+    assert(node_queue_is_empty(&q));
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
 	GD_rank(Root)[i].valid = false;
 	if (GD_flip(g) && GD_rank(g)[i].n > 0) {
@@ -1290,10 +1342,11 @@ void build_ranks(graph_t *g, int pass, ints_t *scratch) {
 
     if (g == dot_root(g) && ncross(scratch) > 0)
 	transpose(g, false);
-    queue_free(&q);
+    node_queue_free(&q);
+    return 0;
 }
 
-void enqueue_neighbors(queue_t *q, node_t *n0, int pass) {
+void enqueue_neighbors(node_queue_t *q, node_t *n0, int pass) {
     edge_t *e;
 
     if (pass == 0) {
@@ -1301,7 +1354,7 @@ void enqueue_neighbors(queue_t *q, node_t *n0, int pass) {
 	    e = ND_out(n0).list[i];
 	    if (!MARK(aghead(e))) {
 		MARK(aghead(e)) = true;
-		queue_push(q, aghead(e));
+		node_queue_push_back(q, aghead(e));
 	    }
 	}
     } else {
@@ -1309,7 +1362,7 @@ void enqueue_neighbors(queue_t *q, node_t *n0, int pass) {
 	    e = ND_in(n0).list[i];
 	    if (!MARK(agtail(e))) {
 		MARK(agtail(e)) = true;
-		queue_push(q, agtail(e));
+		node_queue_push_back(q, agtail(e));
 	    }
 	}
     }
@@ -1376,7 +1429,7 @@ static void flat_reorder(graph_t * g)
 		flat_e = ND_flat_out(v).list[j];
 		if (constraining_flat_edge(g, flat_e)) local_out_cnt++;
 	    }
-	    if ((local_in_cnt == 0) && (local_out_cnt == 0))
+	    if (local_in_cnt == 0 && local_out_cnt == 0)
 		nodes_append(&temprank, v);
 	    else {
 		if (!MARK(v) && local_in_cnt == 0) {
@@ -1399,8 +1452,8 @@ static void flat_reorder(graph_t * g)
 		v = GD_rank(g)[r].v[i];
 		if (ND_flat_out(v).list) {
 		    for (size_t j = 0; (e = ND_flat_out(v).list[j]); j++) {
-			if ( (!GD_flip(g) && ND_order(aghead(e)) < ND_order(agtail(e))) ||
-				 ( (GD_flip(g)) && (ND_order(aghead(e)) > ND_order(agtail(e)) ))) {
+			if ((!GD_flip(g) && ND_order(aghead(e)) < ND_order(agtail(e))) ||
+				  (GD_flip(g) && ND_order(aghead(e)) > ND_order(agtail(e)))) {
 			    assert(!constraining_flat_edge(g, e));
 			    delete_flat_edge(e);
 			    j--;
@@ -1519,11 +1572,11 @@ static int local_cross(elist l, int dir)
     return cross;
 }
 
-static int rcross(graph_t *g, int r, ints_t *Count) {
-    int top, bot, cross, max, i, k;
+static int64_t rcross(graph_t *g, int r, ints_t *Count) {
+    int top, bot, max, i, k;
     node_t **rtop, *v;
 
-    cross = 0;
+    int64_t cross = 0;
     max = 0;
     rtop = GD_rank(g)[r].v;
 
@@ -1564,17 +1617,17 @@ static int rcross(graph_t *g, int r, ints_t *Count) {
     return cross;
 }
 
-static int ncross(ints_t *scratch) {
+static int64_t ncross(ints_t *scratch) {
     assert(scratch != NULL);
-    int r, count, nc;
+    int r;
 
     graph_t *g = Root;
-    count = 0;
+    int64_t count = 0;
     for (r = GD_minrank(g); r < GD_maxrank(g); r++) {
 	if (GD_rank(g)[r].valid)
 	    count += GD_rank(g)[r].cache_nc;
 	else {
-	    nc = GD_rank(g)[r].cache_nc = rcross(g, r, scratch);
+	    const int64_t nc = GD_rank(g)[r].cache_nc = rcross(g, r, scratch);
 	    count += nc;
 	    GD_rank(g)[r].valid = true;
 	}
@@ -1594,8 +1647,7 @@ static int ordercmpf(const void *x, const void *y) {
   return 0;
 }
 
-/* flat_mval:
- * Calculate a mval for nodes with no in or out non-flat edges.
+/* Calculate a mval for nodes with no in or out non-flat edges.
  * Assume (ND_out(n).size == 0) && (ND_in(n).size == 0)
  * Find flat edge a->n where a has the largest order and set
  * n.mval = a.mval+1, assuming a.mval is defined (>=0).
@@ -1833,8 +1885,8 @@ static void mincross_options(graph_t * g)
 
     p = agget(g, "mclimit");
     if (p && (f = atof(p)) > 0.0) {
-	MinQuit = MAX(1, MinQuit * f);
-	MaxIter = MAX(1, MaxIter * f);
+	MinQuit = MAX(1, scale_clamp(MinQuit, f));
+	MaxIter = MAX(1, scale_clamp(MaxIter, f));
     }
 }
 
