@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -16,7 +16,7 @@
 //                                   // 
 ///////////////////////////////////////
 
-#include <cgraph/alloc.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,6 +25,7 @@
 #include <assert.h>
 #include <common/arith.h>
 #include <topfish/hierarchy.h>
+#include <util/alloc.h>
 
 /////////////////////////
 // Some utilities for  //
@@ -127,14 +128,14 @@ static double ddist(ex_vtx_data * geom_graph, int v, int u)
     return hypot(x_v - x_u, y_v - y_u);
 }
 
-extern void quicksort_place(double *, int *, int first, int last);
+extern void quicksort_place(double *, int *, size_t size);
 
 static int 
 maxmatch(v_data * graph,	/* array of vtx data for graph */
 	ex_vtx_data * geom_graph,	/* array of vtx data for graph */
 	int nvtxs,	/* number of vertices in graph */
 	int *mflag,	/* flag indicating vtx selected or not */
-	int dist2_limit
+	bool dist2_limit
     )
 /* 
     Compute a matching of the nodes set. 
@@ -225,7 +226,7 @@ maxmatch(v_data * graph,	/* array of vtx data for graph */
 	matchability[vtx] += min_edge_len;	// we less want to match distant nodes
 	matchability[vtx] += ((double) rand()) / RAND_MAX;	// add some randomness 
     }
-    quicksort_place(matchability, order, 0, nvtxs - 1);
+    quicksort_place(matchability, order, nvtxs);
     free(matchability);
 
     // Start determining the matched pairs
@@ -571,7 +572,7 @@ coarsen_match (
     int *cgeom_nedges,	/* number of edges in coarsened geom_graph */
     int **v2cvp,	/* reference from vertices to coarse vertices */
     int **cv2vp,	/* reference from vertices to coarse vertices */
-    int dist2_limit
+    bool dist2_limit
 )
 
 /*
@@ -609,36 +610,6 @@ coarsen_match (
 			     cnvtxs, v2cv, cv2v);
 }
 
-/* release:
- * Free memory resources for hierarchy.
- */
-void release(Hierarchy * hierarchy)
-{
-    v_data *graph;
-    ex_vtx_data *ex_graph;
-    int i;
-    for (i = 0; i < hierarchy->nlevels; i++) {
-	graph = hierarchy->graphs[i];
-	ex_graph = hierarchy->geom_graphs[i];
-	freeGraph (graph);
-	free(ex_graph[0].edges);
-	free(ex_graph);
-	if (i < hierarchy->nlevels - 1) {
-	    free(hierarchy->v2cv[i]);
-	}
-	if (i > 0) {
-	    free(hierarchy->cv2v[i]);
-	}
-    }
-
-    free(hierarchy->graphs);
-    free(hierarchy->geom_graphs);
-    free(hierarchy->nvtxs);
-    free(hierarchy->nedges);
-    free(hierarchy->cv2v);
-    free(hierarchy->v2cv);
-}
-
 static v_data *cpGraph(v_data * graph, int n, int nedges)
 {
     float *ewgts = NULL;
@@ -692,10 +663,9 @@ static ex_vtx_data *cpExGraph(ex_vtx_data * graph, int n, int nedges)
     return cpGraph;
 }
 
-Hierarchy *create_hierarchy(v_data * graph, int nvtxs, int nedges,
-			    ex_vtx_data * geom_graph, int ngeom_edges,
-			    hierparms_t* parms)
-{
+Hierarchy *create_hierarchy(v_data *graph, int nvtxs, int nedges,
+                            ex_vtx_data *geom_graph, int ngeom_edges,
+                            bool dist2_limit) {
     int cur_level;
     Hierarchy *hierarchy = gv_alloc(sizeof(Hierarchy));
     int cngeom_edges = ngeom_edges;
@@ -742,7 +712,7 @@ Hierarchy *create_hierarchy(v_data * graph, int nvtxs, int nedges,
 	     &hierarchy->nvtxs[cur_level + 1],
 	     &hierarchy->nedges[cur_level + 1], &cngeom_edges,
 	     &hierarchy->v2cv[cur_level], &hierarchy->cv2v[cur_level + 1],
-             parms->dist2_limit);
+	     dist2_limit);
     }
 
     hierarchy->nlevels = cur_level + 1;
@@ -783,42 +753,31 @@ void
 set_active_levels(Hierarchy * hierarchy, int *foci_nodes, int num_foci,
     levelparms_t* parms)
 {
-    int n, i;
-    int *nodes;
-    double *distances;
-    ex_vtx_data *graph;
-    int level;
-    int group_size;
-    int thresh;
-    int vtx;
-    ex_vtx_data *cgraph;
-    int *cv2v;
-    int v, u;
     int min_level = 0;
 
-    graph = hierarchy->geom_graphs[min_level];	// finest graph
-    n = hierarchy->nvtxs[min_level];
+    ex_vtx_data *graph = hierarchy->geom_graphs[min_level]; // finest graph
+    int n = hierarchy->nvtxs[min_level];
 
     // compute distances from foci nodes
-    nodes = gv_calloc(n, sizeof(int));
-    distances = gv_calloc(n, sizeof(double));
-    for (i = 0; i < n; i++) {
+    int *nodes = gv_calloc(n, sizeof(int));
+    double *distances = gv_calloc(n, sizeof(double));
+    for (int i = 0; i < n; i++) {
 	nodes[i] = i;
 	distances[i] = dist_from_foci(graph, i, foci_nodes, num_foci);
     }
 
     // sort nodes according to their distance from foci
-    quicksort_place(distances, nodes, 0, n - 1);
+    quicksort_place(distances, nodes, n);
 
     /* compute *desired* levels of fine nodes by distributing them into buckets
      * The sizes of the buckets is a geometric series with 
      * factor: 'coarsening_rate'
      */
-    level = min_level;
-    group_size = parms->num_fine_nodes * num_foci;
-    thresh = group_size;
-    for (i = 0; i < n; i++) {
-	vtx = nodes[i];
+    int level = min_level;
+    int group_size = parms->num_fine_nodes * num_foci;
+    int thresh = group_size;
+    for (int i = 0; i < n; i++) {
+	const int vtx = nodes[i];
 	if (i > thresh && level < hierarchy->nlevels - 1) {
 	    level++;
 	    group_size = (int) (group_size * parms->coarsening_rate);
@@ -835,13 +794,13 @@ set_active_levels(Hierarchy * hierarchy, int *foci_nodes, int num_foci,
     // and 'v' are merged, than the active level of '{u,v}' will be 
     // the minimum of the active levels of 'u' and 'v'
     for (level = min_level + 1; level < hierarchy->nlevels; level++) {
-	cgraph = hierarchy->geom_graphs[level];
+	ex_vtx_data *const cgraph = hierarchy->geom_graphs[level];
 	graph = hierarchy->geom_graphs[level - 1];
-	cv2v = hierarchy->cv2v[level];
+	const int *const cv2v = hierarchy->cv2v[level];
 	n = hierarchy->nvtxs[level];
-	for (i = 0; i < n; i++) {
-	    v = cv2v[2 * i];
-	    u = cv2v[2 * i + 1];
+	for (int i = 0; i < n; i++) {
+	    const int v = cv2v[2 * i];
+	    const int u = cv2v[2 * i + 1];
 	    if (u >= 0) {	// cv is decomposed from 2 fine nodes
 		if (graph[v].active_level < level
 		    || graph[u].active_level < level) {
@@ -866,17 +825,17 @@ set_active_levels(Hierarchy * hierarchy, int *foci_nodes, int num_foci,
     //----------------------
     // Propagate final levels all the way to fine nodes
     for (level = hierarchy->nlevels - 1; level > 0; level--) {
-	cgraph = hierarchy->geom_graphs[level];
+	ex_vtx_data *const cgraph = hierarchy->geom_graphs[level];
 	graph = hierarchy->geom_graphs[level - 1];
-	cv2v = hierarchy->cv2v[level];
+	const int *const cv2v = hierarchy->cv2v[level];
 	n = hierarchy->nvtxs[level];
-	for (i = 0; i < n; i++) {
+	for (int i = 0; i < n; i++) {
 	    if (cgraph[i].active_level < level) {
 		continue;
 	    }
 	    // active level has been already reached, copy level to children
-	    v = cv2v[2 * i];
-	    u = cv2v[2 * i + 1];
+	    const int v = cv2v[2 * i];
+	    const int u = cv2v[2 * i + 1];
 	    graph[v].active_level = cgraph[i].active_level;
 	    if (u >= 0) {
 		graph[u].active_level = cgraph[i].active_level;
